#!/usr/bin/env ruby
# -*- coding: utf-8 -*-
$: << './lib/'
require 'steppable'
require 'pry'

example_step 1, 'Just debug it' do
  require 'csv'
  class Product < Struct.new(:name, :price, :category)
  end

  csv = CSV.read('./products.csv', encoding: 'UTF-8')
  File.open('/tmp/data.txt', 'w') do |f|
    csv.each do |row|
      product = Product.new(*row)
      # will raise here
      # p product

      f.puts "%s: %d" % [product.name, product.price]
    end
  end
end

example_step 2, 'RSpec interruption' do
  require 'rspec'
  RSpec::Core::Runner.autorun
  describe "plus" do
    subject { 1 + 2 }
    it { should == 4 } # somewhat fails...
    it { binding.pry } # check!
  end
end

example_step 3, 'Catch exceptions with pry' do
  require 'pry-exception_explorer'
  PryExceptionExplorer.enabled = true
  PryExceptionExplorer.intercept(ArgumentError)

  def foo(n)
    raise ArgumentError if n < 0
    n ** 2
  end

  foo(-1) # raise!
end

example_step 4, 'Pry-remote sample' do
  require 'pry-remote'

  class Foo
    def initialize(x, y)
      @x = x
      @y = y
      @z = @x ** @y
      binding.remote_pry
    end
  end
  Foo.new(2, 3)
end

example_step 5, 'Power of webconsole sample with sinatra' do
  require 'sinatra'
  require 'rack-webconsole'
  require 'cgi'
  configure :development do
    use Rack::Webconsole
    Rack::Webconsole.inject_jquery = true
    Rack::Webconsole.key_code = [96]
  end

  get '/' do
    @foo_value = settings.foo_value rescue nil
    erb <<EOT
<!DOCTYPE html>
<html>
<head><title>Rack::Webconsole sample</title></head>
<body>
<h1>Rack::Webconsole sample</h1>
<pre style="font-family: Courier, monospace;">
This is a sample app for Rack::Webconsole
Please type ` (backquote) and in the web terminal,
try the command below:

    Sinatra::Application.set :foo_value, "Some Text"

and then reload me...

<strong>settings.foo_value = <%= CGI.escapeHTML @foo_value.inspect %></strong>

You even can add routes by sinatra verb!
</pre>
</body>
</html>
EOT
  end
end

example_step 6, 'Pry command sample' do
  Pry::Commands.block_command "fizz-buzz", "The command desc" do |limit|
    (1..limit.to_i).each do |i|
      case
      when i % 3 == 0 && i % 5 == 0; output.puts "FizzBuzz"
      when i % 3 == 0; output.puts "Fizz"
      when i % 5 == 0; output.puts "Buzz"
      else; output.puts i
      end
    end
  end
  puts "Let's try the command 'fizz-buzz 10'!"
  Pry.start
end
